/*IR-Receiver und Dekode, Revision:00
  Copyright: Ing. M. Cech, elektronische Gerte, A-2432 Schwadorf

  enthlt i/o-function

  Autor: M. Cech
  (C) Ing. M. Cech, elektronische Gerte
  Lagerhausstr.12, A-2432 Schwadorf

  Datum: 21/08/2004
  Rev.:00

  Compiler options: Strings in flash only, small model of printf, 8515

  Revision history:

*/
//ICC-AVR application builder : 11.04.01 20:50:15

// Target: AT90S8535

//ICC needs these helper files...
#include <iom8535v.h>
#include <macros.h>
#include "Muster_BaseType.h"
#include "Muster_glob.h"
#include "Muster_Prot.h"


//------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------

void port_init(void)  /* */
{
 PORTA=0x07;
 DDRA=0xF8;
 PORTB=0xFF;
 DDRB=0x00;
 PORTC=0xFF;
 DDRC=0x00;
 PORTD=0x1F;
 DDRD=0xE0;
}

//------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------
//TIMER0 initialize - prescale:64
// Target : M8535
// Crystal: 16.000Mhz
// WGM: Normal
// desired value: 1000Hz
// actual value: 1000,000Hz (0,0%)
void timer0_init(void)
{
 TCCR0 = 0x00; //stop
 TCNT0 = 0x06; //set count
 OCR0  = 0xFA;  //set compare
 TCCR0 = 0x03; //start timer
}
//------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------

void timer1_init(void)  /* */
{
 TCCR1B = 0x00; //stop
 TCNT1H = 0x00 /*Initial setting*/; //setup
 TCNT1L = 0x00 /*Initial setting*/;
 OCR1AH = 0x00 /*Initial setting*/;
 OCR1AL = 0x00 /*Initial setting*/;
 OCR1BH = 0x00 /*Initial setting*/;
 OCR1BL = 0x00 /*Initial setting*/;
 TCCR1A = 0x00;
 TCCR1B = 0x00; //start Timer CLK=1/8, frequency = 0Hz
}

//------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------

//UART0 initialize
// Target : M8535
// Crystal: 16.000Mhz
// desired baud rate: 9600
// actual: baud rate:9615 (0,2%)
void uart0_init(void)
{
 UCSRB = 0x00; //disable while setting baud rate
 UCSRA = 0x00;
 UCSRC = BIT(URSEL) | 0x06;
 UBRRL = 0x00; //set baud rate lo
 UBRRH = 0x00; //set baud rate hi
 UCSRB = 0x98;
}

//------------------------------------------------------------------------------------------------------------
//------------------------------------------------------------------------------------------------------------

//call this routine to initialise all peripherals
void init_devices(void)  /* */
{
 //stop errant interrupts until set up
 CLI(); //disable all interrupts

 port_init();
 timer0_init();
 uart0_init();


 MCUCR = 0x00;
 GICR  = 0x00;
 TIMSK = 0x01; //timer interrupt sources
 SEI(); //re-enable interrupts
 //all peripherals are now initialised
}

//****************************************************************************************************
//****************************************************************************************************

//read a byte from the LCD data register
//control lines: LCD_Enable, LCD_RD//WR, LCD_Register_Sel
//LCD latches databus contents at rising signal of E (LCD_Enable)
//Data direction is adjusted by R//W (LCD_RD//WR)
//Address the control reg by accessing RS (LCD_Register_Sel) RS=1 >> address Register

UInt8 rd_LCD(void)
{
 UInt8 var2;

 DDRC=0x00;		// Adjust data direction of Port C: set to input

 PORTD=0x40;		// E=0, R//W=1, RS=0
 PORTD=0xC0;		// E=1, R//W=1, RS=0
 PORTD=0xC0;		// E=1, R//W=1, RS=0

 var2=PINC;	// read contents at the bus taps

 PORTD=0x40;		// E=0, R//W=1, RS=0

 return(var2);
}

//****************************************************************************************************
//****************************************************************************************************

//test wether LCD-transaction is finished, use LCD-Busy flag in control register
//and wait until finished

void LCD_busy_test(void)

{
 while (rd_LCD()&0x80); //Bit 7 is the busy indicator of the LCD

}

//****************************************************************************************************
//****************************************************************************************************

//switch LCD-Backlight LED ON or OFF

void LCD_LED(UInt8 c)
{

 DDRA=0xF8;	// Adjust data direction of Port A: set all except 0,1,2 to output
 if (c)
  PORTA|=0x80;		// set PA7
 else
  PORTA&=0x7F;		// clear PA7

}


//****************************************************************************************************
//****************************************************************************************************

//write a single character to the LCD

void LCD_putc(UInt8 c)
{
 LCD_busy_test();
 if (c == '\n')
  wr_LCD_control(LCD_NEW_LINE);
 else
  wr_LCD(c);
}


//****************************************************************************************************
//****************************************************************************************************

//write a \0 terminated string to the LCD at a previously defined address

void LCD_puts_const(const UInt8 *s)
{
 while (*s) {
 LCD_busy_test();
 if (*s == '\n')
  wr_LCD_control(LCD_NEW_LINE);
 else
  wr_LCD(*s);
  ++s;
 }
}
//****************************************************************************************************
//****************************************************************************************************

//write a \0 terminated string to the LCD at a previously defined address

void LCD_puts(UInt8 *s)
{
 while (*s) {
 LCD_busy_test();
 if (*s == '\n')
  wr_LCD_control(LCD_NEW_LINE);
 else
  wr_LCD(*s);
  ++s;
 }
}

//****************************************************************************************************
//****************************************************************************************************

//write a command to the LCD

void LCD_command(UInt8 c)
{
 LCD_busy_test();
 wr_LCD_control(c);
}


//****************************************************************************************************
//****************************************************************************************************

void LCD_init(void)							// UPH: Initialize the LCD.
{
UInt8 i;

 for (i=0; i<5; i++) {
  LCD_busy_test();
  wr_LCD_control(init_tab[i]);
 }
}

//****************************************************************************************************
//****************************************************************************************************

void LCD_reset(void)							// UPH: Initialize the LCD.
{

 T0_Var1=15;
 while(T0_Var1);
 wr_LCD_control(0x30);
 T0_Var1=5;
 while(T0_Var1);
 wr_LCD_control(0x30);
 T0_Var1=1;
 while(T0_Var1);
 wr_LCD_control(0x30);
}

//****************************************************************************************************
//****************************************************************************************************

//write a byte to the LCD data register
//control lines: LCD_Enable, LCD_RD//WR, LCD_Register_Sel
//LCD latches databus contents at rising signal of E (LCD_Enable)
//Data direction is adjusted by R//W (LCD_RD//WR)
//Address the control reg by accessing RS (LCD_Register_Sel) RS=1 >> address Register

void wr_LCD(UInt8 data)
{

 //strobe data to LCD
 // prepare control lines of LCD
 PORTD=0x20;		// E=0, R//W=0, RS=1
 DDRC=0xFF;		// Adjust data direction of Port C: set to output

 PORTC=data;	// write contents to bus taps

 PORTD=0xA0;		// E=1, R//W=0, RS=1
 PORTD=0x20;		// E=0, R//W=0, RS=1


}

//****************************************************************************************************
//****************************************************************************************************

//write a byte to the LCD control register
//control lines: LCD_Enable, LCD_RD//WR, LCD_Register_Sel
//LCD latches databus contents at rising signal of E (LCD_Enable)
//Data direction is adjusted by R//W (LCD_RD//WR)
//Address the control reg by accessing RS (LCD_Register_Sel) RS=1 >> address Register

void wr_LCD_control(UInt8 data)
{
 // prepare control lines of LCD
 PORTD=0x00;		// E=0, R//W=0, RS=0
 DDRC=0xFF;		// Adjust data direction of Port C: set to output

 PORTC=data;	// write contents to bus taps

 PORTD=0x80;		// E=1, R//W=0, RS=0
 PORTD=0x00;		// E=0, R//W=0, RS=0


}

//****************************************************************************************************
//****************************************************************************************************


void write_4dig_hex_to_LCD(const UInt16 i)  /* */
{
UInt8 ch;

 /* First digit */
 if((ch=(((UInt8 *)&i)[1])>>4) < 10)
   LCD_putc(ch+'0');
 else
   LCD_putc(ch+('A'-10));
 /* Second digit */
 if((ch=(((UInt8 *)&i)[1])&0x0F) < 10)
   LCD_putc(ch+'0');
 else
   LCD_putc(ch+('A'-10));
/* Third digit */
 if((ch=(((UInt8 *)&i)[0])>>4) < 10)
   LCD_putc(ch+'0');
 else
   LCD_putc(ch+('A'-10));
 /* Fourth digit */
 if((ch=(((UInt8 *)&i)[0])&0x0F) < 10)
   LCD_putc(ch+'0');
 else
   LCD_putc(ch+('A'-10));

}

//****************************************************************************************************
//****************************************************************************************************
